@@ -3,23 +3,26 @@ package Params::ValidationCompiler::Compiler;
 use strict;
 use warnings;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
-use Eval::Closure;
-use List::SomeUtils qw( first_index );
+use Eval::Closure qw( eval_closure );
 use Params::ValidationCompiler::Exceptions;
 use Scalar::Util qw( blessed looks_like_number reftype );
 use overload ();
 
 BEGIN {
-    unless (
-        eval {
-            require Sub::Util;
-            Sub::Util->VERSION(1.40);
-            Sub::Util->import('set_subname');
-            1;
-        }
-        ) {
+    ## no critic (Variables::RequireInitializationForLocalVars)
+    local $@;
+    my $has_sub_util = eval {
+        require Sub::Util;
+        Sub::Util->VERSION(1.40);
+        Sub::Util->import('set_subname');
+        1;
+    };
+
+    sub HAS_SUB_UTIL () {$has_sub_util}
+
+    unless ($has_sub_util) {
         *set_subname = sub {
             die
                 "Cannot name a generated validation subroutine. Please install Sub::Util.\n";
@@ -27,7 +30,7 @@ BEGIN {
     }
 }
 
-my %known = map { $_ => 1 } qw( name params slurpy );
+my %known = map { $_ => 1 } qw( name name_is_optional params slurpy );
 
 # I'd rather use Moo here but I want to make things relatively high on the
 # CPAN river like DateTime use this distro, so reducing deps is important.
@@ -76,6 +79,8 @@ sub new {
 sub name      { $_[0]->{name} }
 sub _has_name { exists $_[0]->{name} }
 
+sub _name_is_optional { $_[0]->{name_is_optional} }
+
 # I have no idea why critic thinks _caller isn't used.
 
 ## no critic (Subroutines::ProhibitUnusedPrivateSubroutines)
@@ -103,6 +108,8 @@ sub subref {
     if ( $self->_has_name ) {
         my $caller = $self->_has_caller ? $self->_caller : caller(1);
         my $name = join '::', $caller, $self->name;
+
+        return $sub if $self->_name_is_optional && !HAS_SUB_UTIL;
         set_subname( $name, $sub );
     }
 
@@ -144,16 +151,20 @@ sub _compile_named_args_check {
         my $qname  = B::perlstring($name);
         my $access = "\$args{$qname}";
 
+        # We check exists $spec->{optional} so as not to blow up on a
+        # restricted hash.
         $self->_add_check_for_required_named_param( $access, $name )
-            unless $spec->{optional} || exists $spec->{default};
+            unless ( exists $spec->{optional} && $spec->{optional} )
+            || exists $spec->{default};
 
         $self->_add_default_assignment(
             $access, $name,
             $spec->{default}
         ) if exists $spec->{default};
 
+        # Same issue with restricted hashes here.
         $self->_add_type_check( $access, $name, $spec )
-            if $spec->{type};
+            if exists $spec->{type} && $spec->{type};
     }
 
     if ( $self->slurpy ) {
@@ -287,7 +298,12 @@ sub _compile_positional_args_check {
 
     my @specs = $self->_munge_and_check_positional_params;
 
-    my $first_optional_idx = first_index { $_->{optional} } @specs;
+    my $first_optional_idx = -1;
+    for my $i ( 0 .. $#specs ) {
+        next unless $specs[$i]{optional};
+        $first_optional_idx = $i;
+        last;
+    }
 
     # If optional params start anywhere after the first parameter spec then we
     # must require at least one param. If there are no optional params then
@@ -660,7 +676,7 @@ Params::ValidationCompiler::Compiler - Object that implements the check subrouti
 
 =head1 VERSION
 
-version 0.12
+version 0.13
 
 =for Pod::Coverage .*
 
